@@ -1,8 +1,8 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 
-## Copyright 2005-2011 by LivingLogic AG, Bayreuth/Germany.
-## Copyright 2005-2011 by Walter Dörwald
+## Copyright 2005-2013 by LivingLogic AG, Bayreuth/Germany.
+## Copyright 2005-2013 by Walter Dörwald
 ##
 ## All Rights Reserved
 ##
@@ -55,12 +55,18 @@ Options
 		If true, errors occuring while the database is read or written will be
 		ignored.
 
-	``-e``, ``--encoding`` : encoding
-		The encoding of the output (if ``-x`` is not given; default is ``utf-8``).
+	``--include`` : regexp
+		Only include objects in the output if their name contains the regular
+		expression.
+
+	``--exclude`` : regexp
+		Exclude objects from the output if their name contains the regular
+		expression.
+
 """
 
 
-import sys, os, argparse
+import sys, os, re, argparse
 
 from ll import misc, astyle, orasql
 
@@ -83,7 +89,8 @@ def main(args=None):
 	p.add_argument("-x", "--execute", dest="execute", help="immediately execute the commands instead of printing them? (default %(default)s)", action=misc.FlagAction, default=False)
 	p.add_argument("-k", "--keepjunk", dest="keepjunk", help="Output objects with '$' in their name? (default %(default)s)", action=misc.FlagAction, default=False)
 	p.add_argument("-i", "--ignore", dest="ignore", help="Ignore errors? (default %(default)s)", default=False, action=misc.FlagAction)
-	p.add_argument("-e", "--encoding", dest="encoding", help="Encoding for output (default %(default)s)", default="utf-8")
+	p.add_argument(      "--include", dest="include", metavar="REGEXP", help="Include only objects whose name contains PATTERN (default: %(default)s)", type=re.compile)
+	p.add_argument(      "--exclude", dest="exclude", metavar="REGEXP", help="Exclude objects whose name contains PATTERN (default: %(default)s)", type=re.compile)
 
 	args = p.parse_args(args)
 
@@ -106,14 +113,16 @@ def main(args=None):
 	def keep(obj):
 		if obj.owner is not None and not isinstance(obj, orasql.ForeignKey):
 			return False
-		if args.keepjunk:
-			return True
-		if "$" in obj.name:
+		if ("$" in obj.name or obj.name.startswith("SYS_EXPORT_SCHEMA_")) and not args.keepjunk:
+			return False
+		if args.include is not None and args.include.search(obj.name) is None:
+			return False
+		if args.exclude is not None and args.exclude.search(obj.name) is not None:
 			return False
 		return True
 
 	ddls = []
-	for (i, obj) in enumerate(connection.iterobjects(mode="drop", schema="user")):
+	for (i, obj) in enumerate(connection.iterobjects(owner=None, mode="drop")):
 		keepdef = keep(obj)
 		# Get DDL
 		ddl = ""
@@ -142,7 +151,7 @@ def main(args=None):
 			if args.execute:
 				ddls.append((obj, ddl))
 			else:
-				stdout.write(ddl.encode(args.encoding))
+				stdout.write(ddl)
 
 	# Execute DDL
 	if args.execute:
@@ -152,7 +161,7 @@ def main(args=None):
 				stderr.writeln("oradrop.py: ", cs, ": dropping #{}/{} ".format(i+1, len(ddls)), s4object(str(obj)))
 			try:
 				cursor.execute(ddl)
-			except orasql.DatabaseError, exc:
+			except orasql.DatabaseError as exc:
 				if not args.ignore or "ORA-01013" in str(exc):
 					raise
 				stderr.writeln("oradrop.py: ", s4error("{}: {}".format(exc.__class__, str(exc).strip())))
